#! /usr/bin/env python

import sys
import os
import argparse
import inspect
import json
import pickle

from spykeutils.progress_indicator import ProgressIndicator
from spykeutils.plugin.analysis_plugin import AnalysisPlugin
from spykeutils.plugin.data_provider import DataProvider
from spykeutils.plugin import io_plugin

# Import DataProvider implementations so they can be loaded...
import spykeutils.plugin.data_provider_stored


try:  # Use command line progressbar if package is available
    import progressbar

    class ProgressIndicatorConsole(ProgressIndicator):
        """ Implements a progress indicator for the CLI """
        def __init__(self):
            widgets = ['', progressbar.Percentage(), ' ',
                       progressbar.Bar(), ' ', progressbar.ETA()]
            self.bar = progressbar.ProgressBar(widgets=widgets)
            self.tick = 0
            self.maxval = 0

        def set_ticks(self, ticks):
            """ Set the required number of ticks before the operation is done """
            self.bar.maxval = ticks
            self.maxval = ticks
            self.tick = 0

        def begin(self, title=''):
            """ Signal that the operation starts """
            self.tick = 0
            self.bar.finished = False
            self.bar.widgets[0] = title + ' '
            self.bar.start()

        def step(self, num_steps=1):
            """ Signal that one or more steps of the operation were completed """
            self.tick += num_steps
            if self.tick > self.bar.maxval:
                self.bar.maxval = self.tick
            self.bar.update(self.tick)
            super(ProgressIndicatorConsole, self).step(num_steps)

        def set_status(self, newStatus):
            """ Set status description """
            self.bar.widgets[0] = newStatus + ' '

        def done(self):
            """ Signal that the operation is done. """
            self.bar.finish()
except ImportError:
    print 'progressbar library not found. Progress of plugin will not be displayed.'
    ProgressIndicatorConsole = ProgressIndicator


def main():
    parser = argparse.ArgumentParser(description='Start an analysis plugin')
    parser.add_argument('File', type=str, help='Parameter file',
                        default='plugin_data')
    parser.add_argument('-d', '--direct', dest='direct',
                        action='store_const', const=True, default=False,
                        help='File is a pickle string instead of a filename')
    parser.add_argument('-dd', '--datadir', type=str, help='The data directory')

    cl_args = parser.parse_args()

    if cl_args.direct:
        args = pickle.loads(cl_args.File)
    else:
        f = open(cl_args.File, 'r')
        args = pickle.load(f)

    exc_globals = {}
    exec(args['file'], exc_globals)

    # Load plugin
    plugin = None
    for cl in exc_globals.values():
        if not inspect.isclass(cl):
            continue

        if not issubclass(cl, AnalysisPlugin):
            continue

        if not cl.__name__ == args['name']:
            continue

        plugin = cl()
        break

    if not plugin:
        sys.stderr.write('Could not find plugin class, aborting...\n')
        sys.exit(1)

    # Load configuration
    if args['config']:
        plugin.set_parameters(pickle.loads(args['config']))

    # Load IO plugins
    if 'io' in args:
        for io in args['io']:
            io_plugin.load_from_string(io)

    # Load selection
    sel_string = args['selection']
    sels = json.loads(sel_string)

    selections = []
    for s in sels:
        selections.append(DataProvider.from_data(s))

    if cl_args.datadir and os.path.isdir(cl_args.datadir):
        AnalysisPlugin.data_dir = cl_args.datadir

    progress = ProgressIndicatorConsole()
    selections[0].progress = progress

    plugin.start(selections[0], selections[1:])

if __name__ == '__main__':
    main()
